import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import com.renegade_software.AlwaysUptodate.*;

/*
This sample code is intended to be used as a demonstration of how
Always Uptodate can be incorporated into an Swing-based Java application.
The source code in this file is free to be used for any purpose.  However,
this program does access the Renegade Software Web Site to check for
updated versions and in the interest of limiting the impact on our server,
we ask that you please not run this program more than a reasonable number
of times to understand how Always Uptodate works.

For information on how to compile and run this sample program, please refer
to the README.txt file.
*/

public class AlwaysUptodateSwingSample extends JFrame
	implements WindowListener, ActionListener
{

private String proxy_server;
private String port_number;
private String user_id;
private String password;

/* This is a simple inner class to show a jdialog stating that user must */
/* restart the application because of the update. */
public class RestartJDialog extends JDialog implements WindowListener, 
	ActionListener
{
private Font default_font;
private FontMetrics default_fm;

private JLabel message_label1;
private JLabel message_label2;
private JButton okay_button;
private JLabel version_label;

public RestartJDialog( JFrame parent, String new_version )
{
	super( parent, "Restart Message", true );

	this.getContentPane().setLayout( null );
	this.setSize( 375, 250 );
	this.setResizable( false );
	this.addWindowListener( this );
	this.setBackground( Color.lightGray );
	this.setResizable( false );
	default_font = new Font( "Dialog", Font.PLAIN, 12 );
	default_fm = this.getFontMetrics( default_font );
	this.setLocation( parent.getLocation().x + ( parent.getSize().width -
		this.getSize().width ) / 2, parent.getLocation().y +
		( parent.getSize().height - this.getSize().height) / 2 );

	/* Add the window components */
	initWindow( new_version );

	show();
}

/* Action handler for our buttons */
public void actionPerformed( ActionEvent event )
{
	if( event.getSource() == okay_button )
		this.dispose();
}

/* Misc window handler methods so we can catch the windowClosing event */
public void windowOpened( WindowEvent event )
{
}

public void windowClosing( WindowEvent event )
{
	this.dispose();
}
public void windowClosed( WindowEvent event )
{
}

public void windowIconified( WindowEvent event )
{
}

public void windowDeiconified( WindowEvent event )
{
}

public void windowActivated( WindowEvent event )
{
}

public void windowDeactivated( WindowEvent event )
{
}

private void initWindow( String new_version )
{
	message_label1 = new JLabel( "Application updated to version " +
		new_version );
	message_label1.setForeground( Color.black );
	message_label1.setBackground( this.getBackground() );
	message_label1.setFont( default_font );
	message_label1.setSize( default_fm.stringWidth(
		message_label1.getText() ) + 5, default_fm.getHeight() + 5 );
	message_label1.setLocation( ( this.getSize().width -
		message_label1.getSize().width ) / 2, 50 );
	this.getContentPane().add( message_label1 );

	message_label2 = new JLabel( "You must restart this application now." );
	message_label2.setForeground( Color.black );
	message_label2.setBackground( this.getBackground() );
	message_label2.setFont( default_font );
	message_label2.setSize( default_fm.stringWidth(
		message_label2.getText() ) + 5, default_fm.getHeight() + 5 );
	message_label2.setLocation( ( this.getSize().width -
		message_label2.getSize().width ) / 2, 75 );
	this.getContentPane().add( message_label2 );

	version_label = new JLabel( "Current Application Version: " +
		new VersionData().getCurrentVersion() );
	version_label.setForeground( Color.red );
	version_label.setBackground( this.getBackground() );
	version_label.setFont( default_font );
	version_label.setSize( default_fm.stringWidth(
		version_label.getText() ) + 5, default_fm.getHeight() + 5 );
		version_label.setLocation( ( this.getSize().width -
		version_label.getSize().width ) / 2, 120 );
	this.getContentPane().add( version_label );

	okay_button = new JButton( "Okay" );
	okay_button.setFont( default_font );
	okay_button.setSize( 80, 25 );
	okay_button.setLocation( ( this.getSize().width -
		okay_button.getSize().width ) / 2, 175 );
	okay_button.addActionListener( this );
	this.getContentPane().add( okay_button );
}

}

/* This is a simple inner class to show a jdialog stating that this */
/* application does not need to be updated. */
public class NoUpdateJDialog extends JDialog
	implements WindowListener, ActionListener
{
private Font default_font;
private FontMetrics default_fm;

private JLabel message_label;
private JButton okay_button;
private JLabel version_label;
private JLabel update_label1;
private JLabel update_label2;

public NoUpdateJDialog( JFrame parent )
{
	super( parent, "No Update Message", true );

	this.getContentPane().setLayout( null );
	this.setSize( 375, 250 );
	this.setResizable( false );
	this.addWindowListener( this );
	this.setBackground( Color.lightGray );
	this.setResizable( false );
	default_font = new Font( "Dialog", Font.PLAIN, 12 );
	default_fm = this.getFontMetrics( default_font );
	this.setLocation( parent.getLocation().x + ( parent.getSize().width -
		this.getSize().width ) / 2, parent.getLocation().y +
		( parent.getSize().height - this.getSize().height) / 2 );

	/* Add the window components */
	initWindow();

	show();
}

/* Action handler for our buttons */
public void actionPerformed( ActionEvent event )
{
	if( event.getSource() == okay_button )
		this.dispose();
}

/* Misc window handler methods so we can catch the windowClosing event */
public void windowOpened( WindowEvent event )
{
}

public void windowClosing( WindowEvent event )
{
	this.dispose();
}
public void windowClosed( WindowEvent event )
{
}

public void windowIconified( WindowEvent event )
{
}

public void windowDeiconified( WindowEvent event )
{
}

public void windowActivated( WindowEvent event )
{
}

public void windowDeactivated( WindowEvent event )
{
}

private void initWindow()
{
	message_label = new JLabel( "This application does not need to be updated" );
	message_label.setForeground( Color.black );
	message_label.setBackground( this.getBackground() );
	message_label.setFont( default_font );
	message_label.setSize( default_fm.stringWidth(
		message_label.getText() ) + 5, default_fm.getHeight() + 5 );
	message_label.setLocation( ( this.getSize().width -
		message_label.getSize().width ) / 2, 50 );
	this.getContentPane().add( message_label );

	version_label = new JLabel( "Current Application Version: 1.00" );
	version_label.setForeground( Color.red );
	version_label.setBackground( this.getBackground() );
	version_label.setFont( default_font );
	version_label.setSize( default_fm.stringWidth(
		version_label.getText() ) + 5, default_fm.getHeight() + 5 );
	version_label.setLocation( ( this.getSize().width -
		version_label.getSize().width ) / 2, 75 );
	this.getContentPane().add( version_label );

	update_label1 = new JLabel( 
		"AlwaysUptodateSwingSample$NoUpdateJDialog.class" );
	update_label1.setForeground( Color.blue );
	update_label1.setBackground( this.getBackground() );
	update_label1.setFont( default_font );
	update_label1.setSize( default_fm.stringWidth(
		update_label1.getText() ) + 5, default_fm.getHeight() + 5 );
	update_label1.setLocation( ( this.getSize().width -
		update_label1.getSize().width ) / 2, 110 );
	this.getContentPane().add( update_label1 );

	update_label2 = new JLabel( "Last Updated: Never" );
	update_label2.setForeground( Color.blue );
	update_label2.setBackground( this.getBackground() );
	update_label2.setFont( default_font );
	update_label2.setSize( default_fm.stringWidth(
		update_label2.getText() ) + 5, default_fm.getHeight() + 5 );
	update_label2.setLocation( ( this.getSize().width -
		update_label2.getSize().width ) / 2, 130 );
	this.getContentPane().add( update_label2 );

	okay_button = new JButton( "Okay" );
	okay_button.setFont( default_font );
	okay_button.setSize( 80, 25 );
	okay_button.setLocation( ( this.getSize().width -
		okay_button.getSize().width ) / 2, 175 );
	okay_button.addActionListener( this );
	this.getContentPane().add( okay_button );
}

}

/* This is an inner class that just stores the current version of the */
/* application.  This is very handy to avoid having to maintain the version */
/* number in your own configuration files. */
/* Just update this class every time you release a new version. */
public class VersionData
{

private String current_version = "1.00";

public VersionData()
{
}

public String getCurrentVersion()
{
	return current_version;
}

}

private Font default_font;
private FontMetrics default_fm;

private JLabel message_label;
private JButton update_button;
private JLabel version_label;
private JLabel update_label1;
private JLabel update_label2;

private String current_version;

public static void main( String[] argv )
{
	AlwaysUptodateSwingSample au;
	String proxy_server;
	String port_number;
	String user_id;
	String password;

	/* We expect either 0 or 4 arguments */
	if( argv.length != 0 )
	{
		if( argv.length != 4 )
		{
			System.out.println( "Usage:" );
			System.out.println( "    No Proxy Server:" );
			System.out.println( "         java -classpath AlwaysUptodate.jar " +
				"AlwaysUptodateAWTSample" );
			System.out.println( "    Proxy Server:" );
			System.out.println( "         java -classpath AlwaysUptodate.jar " +
				"AlwaysUptodateAWTSample proxy_server port_number user_id password" );
			System.exit( 0 );
		}

		proxy_server = argv[0];
		port_number = argv[1];
		user_id = argv[2];
		password = argv[3];
	}
	else
	{
		proxy_server = null;
		port_number = null;
		user_id = null;
		password = null;
	}

	au = new AlwaysUptodateSwingSample( proxy_server, port_number, user_id,
		password );
}

public AlwaysUptodateSwingSample( String proxy_server, String port_number,
	String user_id, String password )
{
	super( "Always Uptodate Swing Sample" );

	Toolkit toolkit;
	VersionData sample_version;

	/* Create a new version object and get the current version */
	/* of this application */
	sample_version = new VersionData();
	current_version = sample_version.getCurrentVersion();

	this.getContentPane().setLayout( null );
	this.addWindowListener( this );
	this.setBackground( Color.lightGray );
	this.setResizable( false );
	default_font = new Font( "Dialog", Font.PLAIN, 12 );
	default_fm = this.getFontMetrics( default_font );
	setSize( 400, 500 );
	toolkit = Toolkit.getDefaultToolkit();
	this.setLocation( ( toolkit.getScreenSize().width / 2 ) -
		( this.getSize().width / 2 ), ( toolkit.getScreenSize().height / 2 ) -
		( this.getSize().height / 2 ));

	this.proxy_server = proxy_server;
	this.port_number = port_number;
	this.user_id = user_id;
	this.password = password;

	/* Add the components */
	initWindow();

	this.show();
}

/* Window Listeners */
public void windowOpened( WindowEvent event )
{
}

public void windowClosing( WindowEvent event )
{
	this.dispose();
	System.exit( 0 );
}
public void windowClosed( WindowEvent event )
{
}

public void windowIconified( WindowEvent event )
{
}

public void windowDeiconified( WindowEvent event )
{
}

public void windowActivated( WindowEvent event )
{
}

public void windowDeactivated( WindowEvent event )
{
}

/* Action Listener */
public void actionPerformed( ActionEvent e )
{
	if( e.getSource() == update_button )
		callAlwaysUptodate();
}

private void initWindow()
{
	message_label = new JLabel( "Click here to try the Always Uptodate " +
		"Swing Sample" );
	message_label.setForeground( Color.black );
	message_label.setBackground( this.getBackground() );
	message_label.setFont( default_font );
	message_label.setSize( default_fm.stringWidth(
		message_label.getText() ) + 5, default_fm.getHeight() + 5 );
	message_label.setLocation( ( this.getSize().width -
		message_label.getSize().width ) / 2, 150 );
	this.getContentPane().add( message_label );

	update_button = new JButton( "Update" );
	update_button.setFont( default_font );
	update_button.setSize( 80, 25 );
	update_button.setLocation( ( this.getSize().width -
	update_button.getSize().width ) / 2, message_label.getLocation().y + 100 );
	update_button.addActionListener( this );
	this.getContentPane().add( update_button );

	version_label = new JLabel( "Current Application Version 1.00" );
	version_label.setForeground( Color.red );
	version_label.setBackground( this.getBackground() );
	version_label.setFont( default_font );
	version_label.setSize( default_fm.stringWidth(
		version_label.getText() ) + 5, default_fm.getHeight() + 5 );
	version_label.setLocation( ( this.getSize().width -
		version_label.getSize().width ) / 2, message_label.getLocation().y + 200 );
	this.getContentPane().add( version_label );

	update_label1 = new JLabel( "AlwaysUptodateAWTSample.class" );
	update_label1.setForeground( Color.blue );
	update_label1.setBackground( this.getBackground() );
	update_label1.setFont( default_font );
	update_label1.setSize( default_fm.stringWidth(
		update_label1.getText() ) + 5, default_fm.getHeight() + 5 );
	update_label1.setLocation( ( this.getSize().width -
		update_label1.getSize().width ) / 2, version_label.getLocation().y + 50 );
	this.getContentPane().add( update_label1 );

	update_label2 = new JLabel( "Last Updated: Never" );
	update_label2.setForeground( Color.blue );
	update_label2.setBackground( this.getBackground() );
	update_label2.setFont( default_font );
	update_label2.setSize( default_fm.stringWidth(
		update_label2.getText() ) + 5, default_fm.getHeight() + 5 );
	update_label2.setLocation( ( this.getSize().width -
		update_label2.getSize().width ) / 2, update_label1.getLocation().y + 20 );
	this.getContentPane().add( update_label2 );
}

/* This is it... basically all you need to add to your application to use */
/* Always Uptodate is here.  Simply create an AlwaysUptodate object, check */
/* to see if an update occurred and exit if the application was updated. */
private void callAlwaysUptodate()
{
	String base_dir;
	String update_dir;
	String update_URL;
	AlwaysUptodate au;

	/* Define the base directory that the application is located in */
	/* This example uses the current working directory.  You can also specify */
	/* a full path like: "c:\app_dir\my_app\" or "/apps/my_app/". */
	base_dir = "./";

	/* Define the temporary location for downloaded files.  Relative paths */
	/* can be specified by using "./".  However, relative paths are relative */
	/* to the base directory, not the current working directory.  You can also */
	/* specify a full path like: "c:\app_dir\my_app\AlwaysUptodate\" or */
	/* "/usr/tmp".  If this directory does not exist, it will be built */
	/* automatically. This example uses a subdirectory called AlwaysUptodate */
	/* under the base directory which happens to be the current working */
	/* directory. */
	update_dir = "./AlwaysUptodate";

	/* Define the URL to the versions data file on your web server.  This file */
	/* is used to first determine if the application is up to date and then */
	/* to identify how many versions out of date it is. */
	update_URL = "http://www.renegade-software.com/AlwaysUptodate/SwingSample/" +
		"versions.dat";

	/* The arguments for creating an AlwaysUptodate object are:           */
	/*   this: pass in a JFrame object since Always Uptodate uses dialog  */
	/*         boxes that need to have a parent jframe to position them   */
	/*         properly.  If you aren't using a JFrame, you can simply    */
	/*         pass "new JFrame()".                                       */
	/*                                                                    */
	/*   "Always Uptodate Sample": this is the application name that will */
	/*                             be displayed by Always Uptodate        */
	/*                                                                    */
	/*   update_URL: this is the URL of the versions file on your web     */
	/*               site                                                 */
	/*                                                                    */
	/*   current_version: this is the current version of this application */
	/*                                                                    */
	/*   base_dir: this is the application's base directory.  Any files   */
	/*             that need to be updated must be located under this     */
	/*             directory.                                             */
	/*                                                                    */
	/*   update_dir: this is the location used for temporary file storage */
	/*               during the update process.                           */
	/*                                                                    */
	/*   proxy_server: this is the proxy server name                       */
	/*                                                                     */
	/*   port_number: this is the port number the proxy server uses        */
	/*                                                                     */
	/*   user_id: this is the user id for connecting to the proxy server   */
	/*                                                                     */
	/*   password: this is the password for connecting to the proxy server */
	/*                                                                     */

	/* Create a new AlwaysUptodate object */
	au = new AlwaysUptodate( this, "Always Uptodate Sample", update_URL,
		current_version, base_dir, update_dir, proxy_server, port_number, user_id,
		password );

	/* Note: To not use a proxy server, you can either pass a null value for */
	/* "proxy_server" or call the alternative constructor:                   */
	/* au = new AlwaysUptodate( this, "Always Uptodate Sample", update_URL,  */
	/* current_version, base_dir, update_dir );                              */

	/* Now we can see the outcome of the update check: */

	/* If an update took place: */
	if( au.getUpdatePerformed() == true )
	{
		/* Warn the user that they have to restart the app */
		new RestartJDialog( this, au.getNewVersion() );

		/* Exit the application */
		System.exit( 0 );
	}

	/* This is optional, but because no update took place, */
	/* You might want to show a dialog to tell the user that their */
	/* application is already uptodate if the update did not fail because of */
	/* an error. */
	/* Always Uptodate does not do this for you since you might want to */
	/* use it in "silent" mode for automatic update checks.  It doesn't */
	/* make sense to display a message telling the user that their */
	/* application doesn't need to be updated if they didn't explictly */
	/* ask for an update check. */
	/* Also, since the user can refuse an update, check for whether the */
	/* user chose not to update before you display a message like this. */
	else if( ( au.getUpdateRefused() == false ) && 
		( au.getErrorOccurred() == false ) )
			new NoUpdateJDialog( this );
}

}
